@@ -31,6 +31,7 @@ from cms.models.permissionmodels import (
 )
 from cms.plugin_rendering import ContentRenderer, StructureRenderer
 from cms.test_utils.util.context_managers import UserLoginContext
+from cms.toolbar.utils import get_toolbar_from_request
 from cms.utils.compat import DJANGO_4_1
 from cms.utils.conf import get_cms_setting
 from cms.utils.permissions import set_current_user
@@ -122,6 +123,7 @@ class BaseCMSTestCase:
             'can_add': False,
             'can_change': False,
             'can_delete': False,
+            'can_publish': False,
             'can_change_advanced_settings': False,
             'can_change_permissions': False,
             'can_move_page': False,
@@ -139,6 +141,7 @@ class BaseCMSTestCase:
             'can_add': False,
             'can_change': False,
             'can_delete': False,
+            'can_publish': False,
             'can_change_advanced_settings': False,
             'can_change_permissions': False,
             'can_move_page': False,
@@ -152,6 +155,18 @@ class BaseCMSTestCase:
         return pp
 
     def get_page_title_obj(self, page, language="en"):
+        import warnings
+
+        from cms.utils.compat.warnings import RemovedInDjangoCMS42Warning
+
+        warnings.warn(
+            "get_page_title_obj is deprecated, use get_pagecontent_obj instead",
+            RemovedInDjangoCMS42Warning,
+            stacklevel=2,
+        )
+        return PageContent.objects.get(page=page, language=language)
+
+    def get_pagecontent_obj(self, page, language="en"):
         return PageContent.objects.get(page=page, language=language)
 
     def _create_user(self, username, is_staff=False, is_superuser=False,
@@ -196,6 +211,7 @@ class BaseCMSTestCase:
         user.user_permissions.add(Permission.objects.get(codename='delete_link'))
         user.user_permissions.add(Permission.objects.get(codename='change_link'))
         # Page permissions
+        user.user_permissions.add(Permission.objects.get(codename='publish_page'))
         user.user_permissions.add(Permission.objects.get(codename='add_page'))
         user.user_permissions.add(Permission.objects.get(codename='change_page'))
         user.user_permissions.add(Permission.objects.get(codename='delete_page'))
@@ -420,7 +436,7 @@ class BaseCMSTestCase:
         else:
             request.current_page = None
 
-        class MockStorage():
+        class MockStorage:
 
             def __len__(self):
                 return 0
@@ -498,7 +514,7 @@ class BaseCMSTestCase:
 
     def get_admin_url(self, model, action, *args):
         opts = model._meta
-        url_name = "{}_{}_{}".format(opts.app_label, opts.model_name, action)
+        url_name = f"{opts.app_label}_{opts.model_name}_{action}"
         return admin_reverse(url_name, args=args)
 
     def get_permissions_test_page(self):
@@ -556,9 +572,9 @@ class BaseCMSTestCase:
 
     def get_add_plugin_uri(self, placeholder, plugin_type, language='en', parent=None, position=None):
         if placeholder.page:
-            path = placeholder.page.get_absolute_url(language)
+            path = placeholder.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         if position is None:
             position = placeholder.get_next_plugin_position(language, parent=parent, insert_order='last')
@@ -580,9 +596,9 @@ class BaseCMSTestCase:
         language = language or 'en'
 
         if plugin.page:
-            path = plugin.page.get_absolute_url(language)
+            path = plugin.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         endpoint = admin_reverse('cms_placeholder_edit_plugin', args=(plugin.pk,))
         endpoint += '?' + urlencode({'cms_path': path})
@@ -592,9 +608,9 @@ class BaseCMSTestCase:
         language = language or 'en'
 
         if plugin.page:
-            path = plugin.page.get_absolute_url(language)
+            path = plugin.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         endpoint = admin_reverse('cms_placeholder_move_plugin')
         endpoint += '?' + urlencode({'cms_path': path})
@@ -604,9 +620,9 @@ class BaseCMSTestCase:
         language = language or 'en'
 
         if plugin.page:
-            path = plugin.page.get_absolute_url(language)
+            path = plugin.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         endpoint = admin_reverse('cms_placeholder_copy_plugins')
         endpoint += '?' + urlencode({'cms_path': path})
@@ -616,9 +632,9 @@ class BaseCMSTestCase:
         language = language or 'en'
 
         if placeholder.page:
-            path = placeholder.page.get_absolute_url(language)
+            path = placeholder.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         endpoint = admin_reverse('cms_placeholder_copy_plugins')
         endpoint += '?' + urlencode({'cms_path': path})
@@ -628,9 +644,9 @@ class BaseCMSTestCase:
         language = language or 'en'
 
         if plugin.page:
-            path = plugin.page.get_absolute_url(language)
+            path = plugin.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         endpoint = admin_reverse('cms_placeholder_delete_plugin', args=(plugin.pk,))
         endpoint += '?' + urlencode({'cms_path': path})
@@ -640,9 +656,9 @@ class BaseCMSTestCase:
         language = language or 'en'
 
         if placeholder.page:
-            path = placeholder.page.get_absolute_url(language)
+            path = placeholder.page.get_absolute_url(language) or f'/{language}/'
         else:
-            path = '/{}/'.format(language)
+            path = f'/{language}/'
 
         endpoint = admin_reverse('cms_placeholder_clear_placeholder', args=(placeholder.pk,))
         endpoint += '?' + urlencode({
@@ -662,6 +678,12 @@ class BaseCMSTestCase:
         plugin = add_plugin(placeholder, plugin_type, language, **plugin_data[plugin_type])
         return plugin
 
+    def _render_placeholder(self, placeholder, context, **kwargs):
+        request = context['request']
+        toolbar = get_toolbar_from_request(request)
+        content_renderer = toolbar.content_renderer
+        return content_renderer.render_placeholder(placeholder, context, **kwargs)
+
 
 class CMSTestCase(BaseCMSTestCase, testcases.TestCase):
     pass
